'''
    Simple socket server using threads
'''
 
import logging
import socket
import sys
import numbers
import signal
from Adafruit_BME280 import *
from thread import *
from datetime import datetime

# Set LOGGING_ENABLED = 0 to disable printing to the log file.
LOGGING_ENABLED = 1
LOGGING_FILENAME = '/root/thp_server/thp_server.log'
logging.basicConfig(filename=LOGGING_FILENAME, format='%(asctime)s %(levelname)s:%(message)s',level=logging.INFO)
 
exttemp_str = ''
rh_str = ''
hecto_str = ''

exttemp = 25.0
rh = 50.0
pascals = 100000.00
hectopascals = 1000.00

HOST = ''   # Symbolic name meaning all available interfaces
PORT = 10001 # Arbitrary non-privileged port
 
s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)

#Setup temp/humidity/pressure sensor BME280
# OSAMPLE = oversampling mode
sensor = BME280(mode=BME280_OSAMPLE_8)

if (LOGGING_ENABLED):
  logging.info('Socket created')
 
#Bind socket to local host and port
try:
    s.bind((HOST, PORT))
except socket.error as msg:
    logging.error('Bind failed. Error Code : ' + str(msg[0]) + ' Message ' + msg[1])
    sys.exit()
     
if (LOGGING_ENABLED):
  logging.info('Socket bind complete')
 
#Start listening on socket
s.listen(10)

if (LOGGING_ENABLED):
  logging.info('Socket now listening')
 
#Function for handling connections. This will be used to create threads
def clientthread(conn,ipaddr,pid):
    #Sending message to connected client
    #conn.send('Welcome to the server. Type something and hit enter\n') #send only takes string
     
    #infinite loop so that function do not terminate and thread do not end.
    while True:
         
        #Receiving from client
        data = conn.recv(1024)
	#Remove carriage return and line feed
	data = data.strip('\n')
	data = data.strip('\r')
	data = data.upper()
	if(data == 'READ?1'):
		get_THP()
        	reply = exttemp_str + ',' + rh_str
		conn.sendall(reply)
		conn.sendall('\r')
	elif (data == 'READ?3'):
		get_THP()
		reply = exttemp_str + ',' + rh_str + ',' + hecto_str
		conn.sendall(reply)
		conn.sendall('\r')
	#else :
		#reply = 'Unknown command ' + data
		#conn.sendall(reply)
		#conn.sendall('\n')
        if not data: 
            break

    if (LOGGING_ENABLED):    
      logging.info('Disconnected from ' + ipaddr + ' : ' + pid) 
    #came out of loop
    conn.close()

# Read Temperature, Humidity, and Barometric Pressure from BME280
def get_THP():
    global exttemp
    global rh
    global pascals
    global hectopascals
    global exttemp_str
    global rh_str
    global hecto_str

    exttemp = sensor.read_temperature()
    rh = sensor.read_humidity()
    pascals = sensor.read_pressure()
    hectopascals = pascals / 100

    exttemp_str = '{0:.2f}'.format(exttemp)
    rh_str = '{0:.1f}'.format(rh)
    hecto_str = '{0:.3f}'.format(hectopascals)
 
#now keep talking with the client
while 1:
    #wait to accept a connection - blocking call
    conn, addr = s.accept()
    if (LOGGING_ENABLED):
      logging.info('Connected with ' + addr[0] + ' : ' + str(addr[1]))
     
    #start new thread takes 1st argument as a function name to be run, second is the tuple of arguments to the function.
    start_new_thread (clientthread ,(conn,str(addr[0]),str(addr[1]),))
 
s.close()
